unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs,
  LCLIntf, LCLType, EpikTimer,
  X3DNodes, X3DLoad,
  CastleControl, CastleSceneCore, CastleVectors, CastleScene, CastleViewport;

type

  { TForm1 }

  TForm1 = class(TForm)
    CastleControlBase1: TCastleControlBase;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure SetFullScreen(FullScreenState: Boolean);
  private

  public

  end;

var
  Form1: TForm1;
  GlobalCard: TX3DRootNode;
  GlobalBoard: TX3DRootNode;

const
  CardSizeX = 1.394230;
  CardSizeY = 2.000000;
  CardSizeZ = 0.006858;

  GridMarginX = 0.1;
  GridMarginY = 0.1;
  GridMarginZ = 0.001;

implementation

{$R *.lfm}

{ TForm1 }

procedure TForm1.SetFullScreen(FullScreenState: Boolean);
begin
  if FullScreenState then
    ShowWindow(Handle, SW_SHOWFULLSCREEN)
  else
    ShowWindow(Handle, SW_SHOWNORMAL);
end;

{

Deck
70386    4 Castle Embereth (ELD) 239
70764   18 Mountain (THB) 285
70271    4 Fervent Champion (ELD) 124
70652    4 Infuriate (THB) 141
70285    4 Robber of the Rich (ELD) 138
69944    4 Scorch Spitter (M20) 159
68576    4 Runaway Steam-Kin (GRN) 115
70636    4 Anax, Hardened in the Forge (THB) 125
69235    4 Light Up the Stage (RNA) 107
70294    2 Torbran, Thane of Red Fell (ELD) 147
70267    4 Embercleave (ELD) 120
69587    4 Jaya's Greeting (WAR) 136

}
function LoadCard(const TextureUrl: String; x, y, z: Single): TCastleScene;
var
  NewCardNode: TX3DRootNode;
  NewCardScene: TCastleScene;
  TextureNode: TImageTextureNode;
  CardPosition: TVector3;
begin
  NewCardScene := TCastleScene.Create(nil);

  NewCardNode := GlobalCard.DeepCopy as TX3DRootNode;
  TextureNode := NewCardNode.FindNodeByName(TImageTextureNode, 'Texture_CardFront', false) as TImageTextureNode;
  TextureNode.SetUrl('artcache/' + TextureUrl + '.jpg');

  CardPosition := NewCardScene.Translation;
  CardPosition := CardPosition + Vector3(x, y, z);
  NewCardScene.Translation := CardPosition;

  NewCardScene.Load(NewCardNode, true);

  Result := NewCardScene;
end;

function LoadBoard(const TextureUrl: String; x, y, z: Single): TCastleScene;
var
  NewBoardNode: TX3DRootNode;
  NewBoardScene: TCastleScene;
  TextureNode: TImageTextureNode;
  BoardScale: TVector3;
begin
  NewBoardScene := TCastleScene.Create(nil);

  NewBoardNode := GlobalBoard.DeepCopy as TX3DRootNode;
  TextureNode := NewBoardNode.FindNodeByName(TImageTextureNode, 'Texture_BoardFront', false) as TImageTextureNode;
  TextureNode.SetUrl('artcache/' + TextureUrl);

  BoardScale := NewBoardScene.Scale;
  BoardScale := Vector3(x, y, z);
  NewBoardScene.Scale := BoardScale;

  NewBoardScene.Load(NewBoardNode, true);

  Result := NewBoardScene;
end;

function LoadMasterCard: TX3DRootNode;
var
  CardNode: TX3DRootNode;
begin
  CardNode := LoadNode('card/card.x3dv');
  Result := CardNode;
end;

function LoadMasterBoard: TX3DRootNode;
var
  BoardNode: TX3DRootNode;
begin
  BoardNode := LoadNode('board/board.x3dv');
  Result := BoardNode;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  Viewport: TCastleViewport;
  Scene: TCastleScene;
  i: Integer;
begin
  GlobalCard := LoadMasterCard;
  GlobalBoard := LoadMasterBoard;

  Viewport := TCastleViewport.Create(Application);
  Viewport.FullSize := true;
  Viewport.AutoCamera := true; // instead of this, you could do "Viewport.Camera.SetView(...)"
  Viewport.AutoNavigation := true; // instead of this, you could do "Viewport.Navigation := ..."
  CastleControlBase1.Controls.InsertFront(Viewport);

  Scene := TCastleScene.Create(Application);
  Scene.Spatial := [ssRendering, ssDynamicCollisions];
  Scene.ProcessEvents := true;

  // Board
  Scene := LoadBoard('angel.jpg', 25, 25, 1);
  Viewport.Items.Add(Scene);

  Scene.Save('debug.x3d');

  // Top Row
  // 70652    4 Infuriate (THB) 141
  Scene := LoadCard('70652', -1.5 * (CardSizeX + GridMarginX), 0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 69235    4 Light Up the Stage (RNA) 107
  Scene := LoadCard('69235', -0.5 * (CardSizeX + GridMarginX), 0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 70267    4 Embercleave (ELD) 120
  Scene := LoadCard('70267',  0.5 * (CardSizeX + GridMarginX), 0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 69587    4 Jaya's Greeting (WAR) 136
  Scene := LoadCard('69587', 1.5 * (CardSizeX + GridMarginX), 0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // Middle Row
  // 70764   18 Mountain (THB) 285
  Scene := LoadCard('70764', -1.5 * (CardSizeX + GridMarginX), -0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 70386    4 Castle Embereth (ELD) 239
  Scene := LoadCard('70386', -0.5 * (CardSizeX + GridMarginX), -0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 70636    4 Anax, Hardened in the Forge (THB) 125
  Scene := LoadCard('70636',  0.5 * (CardSizeX + GridMarginX), -0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 70285    4 Robber of the Rich (ELD) 138
  Scene := LoadCard('70285',  1.5 * (CardSizeX + GridMarginX), -0.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);


  // Bottom Row
  // 68576    4 Runaway Steam-Kin (GRN) 115
  Scene := LoadCard('68576', -1.5 * (CardSizeX + GridMarginX), -1.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 70294    2 Torbran, Thane of Red Fell (ELD) 147
  Scene := LoadCard('70294', -0.5 * (CardSizeX + GridMarginX), -1.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 70271    4 Fervent Champion (ELD) 124
  Scene := LoadCard('70271',  0.5 * (CardSizeX + GridMarginX), -1.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // 69944    4 Scorch Spitter (M20) 159
  Scene := LoadCard('69944',  1.5 * (CardSizeX + GridMarginX), -1.5 * (CardSizeY + GridMarginY), (CardSizeZ + GridMarginZ));
  Viewport.Items.Add(Scene);

  // Dummy Hand
  // 70764   18 Mountain (THB) 285
  for i := 0 to 6 do
    begin
      Scene := LoadCard('70764', (-2.5 * CardSizeX) + (i * 0.5), -3.0 * CardSizeY, (i + 1) * (CardSizeZ + GridMarginZ));
      Viewport.Items.Add(Scene);
    end;

  // Dummy Deck
  // 70764   18 Mountain (THB) 285
  for i := 0 to 42 do
    begin
      Scene := LoadCard('70764', -3.75 * CardSizeX, -3.0 * CardSizeY, (i+1) * (CardSizeZ + GridMarginZ));
      Scene.Rotation := Vector4(0, 1, 0, Pi);
      Viewport.Items.Add(Scene);
    end;

  Viewport.Items.MainScene := Scene;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  SetFullScreen(True);
  Application.ProcessMessages;
end;

end.

